#include <LiquidCrystal.h> //Character LCD Library include

//LCD Connections are GND/VCC/V0(Contrast pot.)/RS/RW(Connect to gnd)/EN/D0/D1/D2/D3/D4/D5/D6/D7/LED+/LED-
const int rs = 9, en = 8, d4 = 7, d5 = 6, d6 = 5, d7 = 4;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);

#define signalPin 2 //Input signal from the UT373, Must be interrupt capable pin (Pin 2 or 3 on arduino Uno, Nano, ProMini)

unsigned long signalMicros = 0; //micros() count at the pulse detection moment
unsigned long lastMicros = 0; //micros() count for the previous pulse
unsigned long periodicTime = 0; //peridic time of the signal, it is the difference between current pulse micros (=signalMicros) and last pulse micros(=lastmicros)
unsigned int RPMValue = 0; //Caluclated RPM Value (Revolutions Per Second) by dividing 60,000,000 over periodicTime variable
float RPSValue = 0.0; //Calculated RPS Value (Revolutions Per Second) by dividing RPMValue by 60.0

char LCDDataBuffer[32]; //LCD Display data are stored in this buffer
char RPSBuffer[10]; //buffer to convert float RPSValue to string

void UT373_Micros();

void setup() {
Serial.begin(115200);
attachInterrupt(digitalPinToInterrupt(signalPin), UT373_Micros, FALLING);
lcd.begin(16, 2);
}

void loop() {
if (micros() - lastMicros > 1000000){ //If no pulses are detected for 1 second (1,000,000 Microseconds)
  lcd.clear();
  lcd.print("NO SIGNAL");
  delay(250);
  Serial.println("NO SIGNAL");
}
else { //if pulses are detected and variables updated from the interrupt function (UT373_Micros())
  RPMValue = 60000000 / periodicTime;
  RPSValue = RPMValue / 60.0;
  lcd.setCursor(0,0);
  sprintf(LCDDataBuffer, "% 6d RPM      ", RPMValue); 
  lcd.print(LCDDataBuffer);
  lcd.setCursor(0,1);
  dtostrf(RPSValue, 6, 1, RPSBuffer);
  sprintf(LCDDataBuffer, "%s RPS", RPSBuffer); 
  lcd.print(LCDDataBuffer);
  Serial.println(RPMValue);

  delay(250);
  
}
  

}

void UT373_Micros(){
  signalMicros = micros(); //Current pulse timestamp
  periodicTime = signalMicros - lastMicros; //periodic time calculation by calculating current pulse timestamp - last pulse timestamp
  lastMicros = signalMicros; //saving current pulse time to be used as a previous pulse time for the next pulse
}

